/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/ContactList.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq;


import org.mobicq.comm.Message;
import org.mobicq.comm.Util;
import org.mobicq.util.ResourceBundle;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotFoundException;
#sijapp target.if SIEMENS#
import com.siemens.mp.game.Sound;
import com.siemens.mp.game.Vibrator;
#sijapp target.end#


public class ContactList implements CommandListener
{


  // Status (all are mutual exclusive)
  public static final long STATUS_AWAY = 0x00000001;
  public static final long STATUS_CHAT = 0x00000020;
  public static final long STATUS_DND = 0x00000002;
  public static final long STATUS_INVISIBLE = 0x00000100;
  public static final long STATUS_NA = 0x00000004;
  public static final long STATUS_OCCUPIED = 0x00000010;
  public static final long STATUS_OFFLINE = 0xFFFFFFFF;
  public static final long STATUS_ONLINE = 0x00000000;


  // Locations of the status images (inside the JAR file)
  private static final String STATUS_AWAY_IMG = "/status_away.png";
  private static final String STATUS_CHAT_IMG = "/status_chat.png";
  private static final String STATUS_DND_IMG = "/status_dnd.png";
  private static final String STATUS_INVISIBLE_IMG = "/status_invisible.png";
  private static final String STATUS_NA_IMG = "/status_na.png";
  private static final String STATUS_OCCUPIED_IMG = "/status_occupied.png";
  private static final String STATUS_OFFLINE_IMG = "/status_offline.png";
  private static final String STATUS_ONLINE_IMG = "/status_online.png";
  private static final String EVENT_PLAIN_MESSAGE_IMG = "/event_plain_message.png";
  private static final String EVENT_URL_MESSAGE_IMG = "/event_url_message.png";


  // Image objects
  public static Image statusAwayImg;
  public static Image statusChatImg;
  public static Image statusDndImg;
  public static Image statusInvisibleImg;
  public static Image statusNaImg;
  public static Image statusOccupiedImg;
  public static Image statusOfflineImg;
  public static Image statusOnlineImg;
  public static Image eventPlainMessageImg;
  public static Image eventUrlMessageImg;


  // Initializer
  static
  {

    // Construct image objects
    try
    {
      ContactList.statusAwayImg = Image.createImage(ContactList.STATUS_AWAY_IMG);
      ContactList.statusChatImg = Image.createImage(ContactList.STATUS_CHAT_IMG);
      ContactList.statusDndImg = Image.createImage(ContactList.STATUS_DND_IMG);
      ContactList.statusInvisibleImg = Image.createImage(ContactList.STATUS_INVISIBLE_IMG);
      ContactList.statusNaImg = Image.createImage(ContactList.STATUS_NA_IMG);
      ContactList.statusOccupiedImg = Image.createImage(ContactList.STATUS_OCCUPIED_IMG);
      ContactList.statusOfflineImg = Image.createImage(ContactList.STATUS_OFFLINE_IMG);
      ContactList.statusOnlineImg = Image.createImage(ContactList.STATUS_ONLINE_IMG);
      ContactList.eventPlainMessageImg = Image.createImage(ContactList.EVENT_PLAIN_MESSAGE_IMG);
      ContactList.eventUrlMessageImg = Image.createImage(ContactList.EVENT_URL_MESSAGE_IMG);
    }
    catch (IOException e)
    {
      // Do nothing
    }

  }


  // Main menu command
  private static Command mainMenuCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "menu"), Command.ITEM, 1);


  /****************************************************************************/


  // Version id numbers
  private long versionId1;
  private int versionId2;


  // Contact items
  private Vector cItems;


  // Group items
  private Vector gItems;


  // Contact list
  private List contactList;


  // Reference to currently selected contact item
  private ContactListContactItem currSelCItem;


  // Constructor
  public ContactList()
  {

    // Try to load contact list from record store
    try
    {
      this.load();
    }
    catch (Exception e)
    {
      this.versionId1 = -1;
      this.versionId2 = -1;
      this.cItems = new Vector();
      this.gItems = new Vector();
    }

    // Initialize contact list
    this.contactList = new List(ResourceBundle.getString("org.mobicq.res.Text", "contact_list"), Choice.IMPLICIT);
    this.contactList.addCommand(ContactList.mainMenuCommand);
    this.contactList.setCommandListener(this);

  }


  // Returns the id number #1 which identifies (together with id number #2) the saved contact list version
  public long getVersionId1()
  {
    return (this.versionId1);
  }


  // Returns the id number #2 which identifies (together with id number #1) the saved contact list version
  public int getVersionId2()
  {
    return (this.versionId2);
  }


  // Returns all contact items as array
  public synchronized ContactListContactItem[] getContactItems()
  {
    ContactListContactItem[] cItems = new ContactListContactItem[this.cItems.size()];
    this.cItems.copyInto(cItems);
    return (cItems);
  }


  // Returns all group items as array
  public synchronized ContactListGroupItem[] getGroupItems()
  {
    ContactListGroupItem[] gItems = new ContactListGroupItem[this.gItems.size()];
    this.gItems.copyInto(gItems);
    return (gItems);
  }


  // Request display of the given alert and the main menu afterwards
  public void activate(Alert alert)
  {
    Mobicq.display.setCurrent(alert, this.contactList);
  }


  // Request display of the main menu
  public void activate()
  {
    Mobicq.display.setCurrent(this.contactList);
  }


  // Tries to load contact list from record store
  private void load() throws Exception, IOException, RecordStoreException
  {

    // Initialize vectors
    this.cItems = new Vector();
    this.gItems = new Vector();

    // Check whether record store exists
    String[] recordStores = RecordStore.listRecordStores();
    boolean exist = false;
    for (int i = 0; i < recordStores.length; i++)
    {
      if (recordStores[i].equals("contactlist"))
      {
        exist = true;
        break;
      }
    }
    if (!exist) throw (new Exception());

    // Open record store
    RecordStore cl = RecordStore.openRecordStore("contactlist", false);

    // Temporary variables
    byte[] buf;
    ByteArrayInputStream bais;
    DataInputStream dis;

    // Get version info from record store
    buf = cl.getRecord(1);
    bais = new ByteArrayInputStream(buf);
    dis = new DataInputStream(bais);
    if (!(dis.readUTF().equals(Mobicq.VERSION))) throw (new IOException());

    // Get version ids from the record store
    buf = cl.getRecord(2);
    bais = new ByteArrayInputStream(buf);
    dis = new DataInputStream(bais);
    this.versionId1 = dis.readLong();
    this.versionId2 = dis.readInt();

    // Read all remaining items from the record store
    int marker = 3;
    while (marker <= cl.getNumRecords())
    {

      // Get type of the next item
      buf = cl.getRecord(marker++);
      bais = new ByteArrayInputStream(buf);
      dis = new DataInputStream(bais);

      // Loop until no more items are available
      while (dis.available() > 0)
      {

        // Get item type
        int type = dis.readInt();

        // Normal contact
        if (type == 0)
        {

          // Get id, group id, UIN and name from the record store
          int id = dis.readInt();
          int group = dis.readInt();
          String uin = dis.readUTF();
          String name = dis.readUTF();

          // Instantiate ContactListContactItem object and add to vector
          ContactListContactItem ci = new ContactListContactItem(id, group, uin, name);
          this.cItems.addElement(ci);

        }
        // Group of contacts
        else if (type == 1)
        {

          // Get id and name from the record store
          int id = dis.readInt();
          String name = dis.readUTF();

          // Instantiate ContactListGroupItem object and add to vector
          ContactListGroupItem gi = new ContactListGroupItem(id, name);
          this.gItems.addElement(gi);

        }

      }

    }

    // Close record store
    cl.closeRecordStore();

  }


  // Save contact list to record store
  private void save() throws IOException, RecordStoreException
  {

    // Try to delete the record store
    try
    {
      RecordStore.deleteRecordStore("contactlist");
    }
    catch (RecordStoreNotFoundException e)
    {
      // Do nothing
    }

    // Create new record store
    RecordStore cl = RecordStore.openRecordStore("contactlist", true);

    // Temporary variables
    byte[] buf;
    ByteArrayOutputStream baos;
    DataOutputStream dos;

    // Add version info to record store
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);
    dos.writeUTF(Mobicq.VERSION);
    buf = baos.toByteArray();
    cl.addRecord(buf, 0, buf.length);

    // Add version ids to the record store
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);
    dos.writeLong(this.versionId1);
    dos.writeInt(this.versionId2);
    buf = baos.toByteArray();
    cl.addRecord(buf, 0, buf.length);

    // Initialize buffer
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);

    // Iterate through all contact items
    for (int i = 0; i < this.cItems.size(); i++)
    {
      ContactListContactItem cItem = (ContactListContactItem) this.cItems.elementAt(i);

      // Add next contact item
      dos.writeInt(0);
      dos.writeInt(cItem.getId());
      dos.writeInt(cItem.getGroup());
      dos.writeUTF(cItem.getUin());
      dos.writeUTF(cItem.getName());

      // Start new record if it exceeds 4096 bytes
      if (baos.size() >= 4096)
      {

        // Save record
        buf = baos.toByteArray();
        cl.addRecord(buf, 0, buf.length);

        // Initialize buffer
        baos = new ByteArrayOutputStream();
        dos = new DataOutputStream(baos);

      }

    }

    // Iterate through all group items
    for (int i = 0; i < this.gItems.size(); i++)
    {
      ContactListGroupItem gItem = (ContactListGroupItem) this.gItems.elementAt(i);

      // Add next group item
      dos.writeInt(1);
      dos.writeInt(gItem.getId());
      dos.writeUTF(gItem.getName());

      // Start new record if it exceeds 4096 bytes
      if (baos.size() >= 4096)
      {

        // Save record
        buf = baos.toByteArray();
        cl.addRecord(buf, 0, buf.length);

        // Initialize buffer
        baos = new ByteArrayOutputStream();
        dos = new DataOutputStream(baos);

      }

    }

    // Save pending record
    if (baos.size() > 0)
    {

      // Save record
      buf = baos.toByteArray();
      cl.addRecord(buf, 0, buf.length);

    }

    // Close record store
    cl.closeRecordStore();

  }


  // Updates the client-side conact list (called when a new roster has been received)
  public synchronized void update(long versionId1, int versionId2, ContactListItem[] items)
  {

    // Save selected contact entry
    this.saveListPosition();

    // Save version numbers
    this.versionId1 = versionId1;
    this.versionId2 = versionId2;

    // Delete temporary contact items
    for (int i = 0; i < this.cItems.size(); i++)
    {
      ContactListContactItem cItem = (ContactListContactItem) this.cItems.elementAt(i);
      if (cItem.isTemporary())
      {
        this.cItems.removeElementAt(i--);
      }
    }

    // Add new contact items and group items
    for (int i = 0; i < items.length; i++)
    {
      if (items[i] instanceof ContactListContactItem)
      {
        if (!this.cItems.contains(items[i]))
        {
          this.cItems.addElement(items[i]);
        }
      }
      else if (items[i] instanceof ContactListGroupItem)
      {
        if (!this.gItems.contains(items[i]))
        {
          this.gItems.addElement(items[i]);
        }
      }
    }

    // Delete non-existant contact items and group items
    for (int i = 0; i < this.cItems.size(); i++)
    {
      boolean delete = true;
      for (int j = 0; j < items.length; j++)
      {
        if (items[j].equals((ContactListItem) this.cItems.elementAt(i)))
        {
          delete = false;
        }
      }
      if (delete)
      {
        this.cItems.removeElementAt(i--);
      }
    }
    for (int i = 0; i < this.gItems.size(); i++)
    {
      boolean delete = true;
      for (int j = 0; j < items.length; j++)
      {
        if (items[j].equals((ContactListItem) this.gItems.elementAt(i)))
        {
          delete = false;
        }
      }
      if (delete)
      {
        this.gItems.removeElementAt(i--);
      }
    }

    // Save new contact list
    try
    {
      this.save();
    }
    catch (Exception e)
    {
      // Do nothing
    }

    // Update visual list
    this.refreshList();

  }


  // Updates the client-side conact list (called when roster is up to date)
  public synchronized void update()
  {

    // Save selected contact entry
    this.saveListPosition();

    // Update visual list
    this.refreshList();

  }


  // Updates the client-side contact list (called when a contact changes status)
  public synchronized void update(String uin, long status)
  {

    // Save selected contact entry
    this.saveListPosition();

    // Update status
    for (int i = 0; i < this.cItems.size(); i++)
    {
      ContactListContactItem cItem = (ContactListContactItem) this.cItems.elementAt(i);
      if (cItem.getUin().equals(uin))
      {
        cItem.setStatus(status);
      }
    }

    // Update visual list
    this.refreshList();

  }


  // Removes a contact list item
  public synchronized void removeContactItem(ContactListContactItem cItem)
  {

    // Save selected contact entry
    this.saveListPosition();

    // Remove given contact item
    this.cItems.removeElement(cItem);

    // Update visual list
    this.refreshList();

  }


  // Adds the given message to the message queue of the contact item identified by the given UIN
  public synchronized void addMessage(Message message)
  {

    // Search for contact entry and add message to message queue
    boolean listed = false;
    for (int i = 0; i < this.cItems.size(); i++)
    {
      ContactListContactItem cItem = (ContactListContactItem) this.cItems.elementAt(i);
      if (cItem.getUin().equals(message.getSndrUin()))
      {
        cItem.addMessage(message);
        listed = true;
      }
    }

    // Create a temporary contact entry if no contact entry could be found
    if (!listed)
    {
      ContactListContactItem cItem = new ContactListContactItem(0, 0, message.getSndrUin(), message.getSndrUin());
      cItem.setTemporary(true);
      cItem.addMessage(message);
      this.cItems.addElement(cItem);
    }

    // Refresh the contact list
    this.refreshList();

    // Notify splash canvas
    Mobicq.mobicq.getSplashCanvasRef().messageAvailable();

    // Notify user
    #sijapp target.if SIEMENS#
    switch (Mobicq.mobicq.getOptionsRef().getMsgNotificationMode())
    {
      case 1:
        Sound.playTone(1000, 1000);
        break;
      case 2:
        Vibrator.triggerVibrator(1000);
        break;
    }
    #sijapp target.end#

  }


  // Save reference to currently selected contact item
  private void saveListPosition()
  {
    if (this.contactList.size() > 0)
    {
      this.currSelCItem = (ContactListContactItem) this.cItems.elementAt(this.contactList.getSelectedIndex());
    }
  }


  // Clears the reference to the currently selected contact item
  public void resetListPosition()
  {
    this.currSelCItem = null;
    this.refreshList();
  }


  // Refreshes the contact list
  public void refreshList()
  {

    // Insertion sort (sort first by status - online/offline/temprary -, then by nick)
    int j;
    for (int i = 1; i < this.cItems.size(); i++)
    {
      j = i;
      ContactListContactItem v = (ContactListContactItem) this.cItems.elementAt(i);
      ContactListContactItem w;
      while ((w = (ContactListContactItem) this.cItems.elementAt(j - 1)).compareTo(v) > 0)
      {
        this.cItems.setElementAt(w, j);
        if (--j == 0) break;
      }
      if (i != j)
      {
        this.cItems.setElementAt(v, j);
      }
    }

    // Add empty elements to visual contact list, if required
    while (this.contactList.size() < this.cItems.size())
    {
      this.contactList.append("", null);
    }
    // Delete elements from visual contact list, if required
    while (this.contactList.size() > this.cItems.size())
    {
      this.contactList.delete(this.contactList.size() - 1);
    }

    // New focus
    boolean focused = false;

    // Iterate over all contact items
    for (int i = 0; i < this.cItems.size(); i++)
    {
      ContactListContactItem cItem = (ContactListContactItem) this.cItems.elementAt(i);

      // Get status image
      Image img = null;
      long status = cItem.getStatus();
      if (cItem.isTemporary())
      {
        img = null;
      }
      else if (status == ContactList.STATUS_AWAY)
      {
        img = ContactList.statusAwayImg;
      }
      else if (status == ContactList.STATUS_CHAT)
      {
        img = ContactList.statusChatImg;
      }
      else if (status == ContactList.STATUS_DND)
      {
        img = ContactList.statusDndImg;
      }
      else if (status == ContactList.STATUS_NA)
      {
        img = ContactList.statusNaImg;
      }
      else if (status == ContactList.STATUS_OCCUPIED)
      {
        img = ContactList.statusOccupiedImg;
      }
      else if (status == ContactList.STATUS_OFFLINE)
      {
        img = ContactList.statusOfflineImg;
      }
      else if (status == ContactList.STATUS_ONLINE)
      {
        img = ContactList.statusOnlineImg;
      }

      // Get event image, if event is enabled
      if (cItem.isPlainMessageAvailable())
      {
        img = ContactList.eventPlainMessageImg;
      }
      else if (cItem.isUrlMessageAvailable())
      {
        img = ContactList.eventUrlMessageImg;
      }

      // Add/update list item
      this.contactList.set(i, cItem.getName(), img);

      // Change selected item
      if (cItem.isPlainMessageAvailable() || cItem.isUrlMessageAvailable())
      {
        if (!focused)
        {
          focused = true;
          this.contactList.setSelectedIndex(i, true);
          this.currSelCItem = cItem;
        }
      }

    }

    // Focus item
    if (!focused && (this.currSelCItem != null) && (this.cItems.indexOf(this.currSelCItem) != -1))
    {
      if (this.cItems.elementAt(this.contactList.getSelectedIndex()) != this.currSelCItem)
      {
        this.contactList.setSelectedIndex(this.cItems.indexOf(this.currSelCItem), true);
      }
    }
    else if (!focused && (this.contactList.size() > 0))
    {
      this.contactList.setSelectedIndex(0, true);
      this.currSelCItem = (ContactListContactItem) this.cItems.elementAt(0);
    }

  }


  // Command listener
  public void commandAction(Command c, Displayable d)
  {

    // Activate main menu
    if (c == ContactList.mainMenuCommand)
    {
      Mobicq.mobicq.getMainMenuRef().activate();
    }
    // Contact item has been selected
    else if (c == List.SELECT_COMMAND)
    {

      // Get selected contact item
      ContactListContactItem currCItem = (ContactListContactItem) this.cItems.elementAt(this.contactList.getSelectedIndex());

      // Activate the contact item menu
      currCItem.activateMenu();

    }

  }


}
