/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/ContactListContactItem.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq;


import java.util.Date;
import java.util.Vector;

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.ImageItem;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;

import org.mobicq.comm.Message;
import org.mobicq.comm.PlainMessage;
import org.mobicq.comm.RequestInfoAction;
import org.mobicq.comm.SendMessageAction;
import org.mobicq.comm.UpdateContactListAction;
import org.mobicq.comm.UrlMessage;
import org.mobicq.comm.Util;
import org.mobicq.util.ResourceBundle;


public class ContactListContactItem extends ContactListItem
{


  // Persistent variables
  private int id;
  private int group;
  private String uin;
  private String name;


  // Transient variables
  private boolean temporary;
  private long status;
  private Vector messages;


  // Message count
  private int outgoingPlainMessagesCnt;


  // Menu
  private Menu menu;


  // Constructor
  public ContactListContactItem(int id, int group, String uin, String name)
  {
    this.id = id;
    this.group = group;
    this.uin = new String(uin);
    this.name = new String(name);
    this.temporary = false;
    this.status = ContactList.STATUS_OFFLINE;
    this.messages = new Vector();
    this.menu = new Menu();
  }


  // Returns the contact item id
  public int getId()
  {
    return (this.id);
  }


  // Sets the contact item id
  public void setId(int id)
  {
    this.id = id;
  }


  // Returns the group item id to which this contact item belongs
  public int getGroup()
  {
    return (this.group);
  }


  // Sets the group item id to which this contact item belongs
  public void setGroup(int group)
  {
    this.group = group;
  }


  // Returns the uin of this contact item
  public String getUin()
  {
    return (new String(this.uin));
  }


  // Sets the uin of this contact item
  public void setUin(String uin)
  {
    this.uin = new String(uin);
  }


  // Returns the name of this contact item
  public String getName()
  {
    return (new String(this.name));
  }


  // Sets the name of this contact item
  public void setName(String name)
  {
    this.name = new String(name);
  }


  // Returns true if this is an temporary contact
  public boolean isTemporary()
  {
    return (this.temporary);
  }


  // Permanent / temporary contact entry
  public void setTemporary(boolean temporary)
  {
    this.temporary = temporary;
  }


  // Returns the status of this contact item
  public long getStatus()
  {
    return (this.status);
  }


  // Sets the status of this contact item
  public void setStatus(long status)
  {
    this.status = Util.translateStatusReceived(status);
  }


  // Returns the number of available messages
  protected synchronized int getNumOfAvailableMessages()
  {
    return (this.messages.size());
  }


  // Returns true if the next available message is a plain message
  // Returns false if no message at all is available, or if the next available
  // message is an URL message
  protected synchronized boolean isPlainMessageAvailable()
  {
    if (this.messages.size() > 0)
    {
      Message m = (Message) this.messages.elementAt(0);
      return (m instanceof PlainMessage);
    }
    else
    {
      return (false);
    }
  }


  // Returns true if the next available message is an URL message
  // Returns false if no message at all is available, or if the next available
  // message is an URL message
  protected synchronized boolean isUrlMessageAvailable()
  {
    if (this.messages.size() > 0)
    {
      Message m = (Message) this.messages.elementAt(0);
      return (m instanceof UrlMessage);
    }
    else
    {
      return (false);
    }
  }


  // Adds a message to the message vector
  protected synchronized void addMessage(Message message)
  {
    this.messages.addElement(message);
  }


  // Activates the contact item menu
  public void activateMenu()
  {
    this.menu.activate();
  }


  // Checks whether some other object is equal to this one
  public boolean equals(Object obj)
  {
    if (!(obj instanceof ContactListContactItem)) return (false);
    ContactListContactItem ci = (ContactListContactItem) obj;
    return (this.uin.equals(ci.getUin()) && (this.temporary == ci.isTemporary()));
  }


  // Compare two contact items (sort by status/nick)
  public int compareTo(ContactListContactItem cItem2)
  {

    // Get status and temporary flag
    long cItem1Status = this.getStatus();
    boolean cItem1IsTemporary = this.isTemporary();
    long cItem2Status = cItem2.getStatus();
    boolean cItem2IsTemporary = cItem2.isTemporary();

    // Compare, return negative int if cItem1 < cItem2, 0 if cItem1 == cItem2, or positive int if cItem1 > cItem2
    if ((cItem1Status != ContactList.STATUS_OFFLINE) && !cItem1IsTemporary)
    {
      if ((cItem2Status != ContactList.STATUS_OFFLINE) && !cItem2IsTemporary)
      {
        return (this.getName().compareTo(cItem2.getName()));
      }
      else
      {
        return (-1);
      }
    }
    else if ((cItem1Status == ContactList.STATUS_OFFLINE) && !cItem1IsTemporary)
    {
      if ((cItem2Status != ContactList.STATUS_OFFLINE) && !cItem2IsTemporary)
      {
        return (1);
      }
      else if ((cItem2Status == ContactList.STATUS_OFFLINE) && !cItem2IsTemporary)
      {
        return (this.getName().compareTo(cItem2.getName()));
      }
      else
      {
        return (-1);
      }
    }
    else if (cItem1IsTemporary)
    {
      if (cItem2IsTemporary)
      {
        return (this.getName().compareTo(cItem2.getName()));
      }
      else
      {
        return (1);
      }
    }
    else
    {
      return (0);
    }

  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  private class Menu implements CommandListener
  {


    // Command listener
    public void commandAction(Command c, Displayable d)
    {

      // Return to contact list
      if (c == MenuUtil.backCommand)
      {
        Mobicq.mobicq.getContactListRef().activate();
      }
      // Message has been closed
      else if (c == MenuUtil.msgCloseCommand)
      {
        if (ContactListContactItem.this.getNumOfAvailableMessages() > 0)
        {
          this.activate();
        }
        else
        {
          Mobicq.mobicq.getContactListRef().update();
          Mobicq.mobicq.getContactListRef().activate();
        }
      }
      // User wants to send a reply
      else if (c == MenuUtil.msgReplyCommand)
      {

        // Select first list element (new message)
        MenuUtil.menuList.setSelectedIndex(0, true);

        // Reset and display textbox for entering messages
        MenuUtil.messageTextbox.setString(null);
        MenuUtil.messageTextbox.removeCommand(MenuUtil.textboxOkCommand);
        MenuUtil.messageTextbox.addCommand(MenuUtil.textboxSendCommand);
        MenuUtil.messageTextbox.setCommandListener(this);
        Mobicq.display.setCurrent(MenuUtil.messageTextbox);

      }
      // Menu item has been selected
      else if (c == List.SELECT_COMMAND)
      {

        switch (MenuUtil.menuList.getSelectedIndex())
        {
          case 0:   // Send plain message

            // Reset and display textbox for entering messages
            MenuUtil.messageTextbox.setString(null);
            MenuUtil.messageTextbox.removeCommand(MenuUtil.textboxOkCommand);
            MenuUtil.messageTextbox.addCommand(MenuUtil.textboxSendCommand);
            MenuUtil.messageTextbox.setCommandListener(this);
            Mobicq.display.setCurrent(MenuUtil.messageTextbox);

            break;
          case 1:   // Send URL message

            // Reset and display textbox for entering messages
            MenuUtil.messageTextbox.setString(null);
            MenuUtil.messageTextbox.removeCommand(MenuUtil.textboxSendCommand);
            MenuUtil.messageTextbox.addCommand(MenuUtil.textboxOkCommand);
            MenuUtil.messageTextbox.setCommandListener(this);
            Mobicq.display.setCurrent(MenuUtil.messageTextbox);

            break;
          case 2:   // Info

            // Display splash canvas
            SplashCanvas wait1 = Mobicq.mobicq.getSplashCanvasRef();
            wait1.setMessage(ResourceBundle.getString("org.mobicq.res.Text", "wait"));
            wait1.setProgress(0);
            Mobicq.display.setCurrent(wait1);

            // Request info from server
            RequestInfoAction act1 = new RequestInfoAction(ContactListContactItem.this.getUin());
            try
            {
              Mobicq.mobicq.getIcqRef().requestAction(act1);
            }
            catch (MobicqException e)
            {
              MobicqException.handleException(e);
              if (e.isCritical()) return;
            }

            // Start timer
            Mobicq.mobicq.getTimerRef().schedule(new SplashCanvas.RequestInfoTimerTask(act1), 1000, 1000);

            break;
          case 3:   // Remove

            // Check whether contact item is temporary
            if (ContactListContactItem.this.isTemporary())
            {

              // Remove this temporary contact item
              Mobicq.mobicq.getContactListRef().removeContactItem(ContactListContactItem.this);

              // Update and activate contact list
              Mobicq.mobicq.getContactListRef().update();
              Mobicq.mobicq.getContactListRef().activate();

            }
            else
            {

              // Display splash canvas
              SplashCanvas wait2 = Mobicq.mobicq.getSplashCanvasRef();
              wait2.setMessage(ResourceBundle.getString("org.mobicq.res.Text", "wait"));
              wait2.setProgress(0);
              Mobicq.display.setCurrent(wait2);

              // Request contact item removal
              UpdateContactListAction act2 = new UpdateContactListAction(ContactListContactItem.this);
              try
              {
                Mobicq.mobicq.getIcqRef().requestAction(act2);
              }
              catch (MobicqException e)
              {
                MobicqException.handleException(e);
                if (e.isCritical()) return;
              }

              // Start timer
              Mobicq.mobicq.getTimerRef().schedule(new SplashCanvas.UpdateContactListTimerTask(act2), 1000, 1000);

            }

            break;
        }

      }
      // Textbox has been closed
      else if ((c == MenuUtil.textboxOkCommand) || (c == MenuUtil.textboxSendCommand))
      {

        // Message has been entered
        if (d == MenuUtil.messageTextbox)
        {

          // Abort if nothing has been entered
          if (MenuUtil.messageTextbox.getString().length() < 1)
          {
            this.activate();
          }

          // Send plain message
          if (MenuUtil.menuList.getSelectedIndex() == 0)
          {

            // Append advertisement?
            boolean appendAdvertisement = false;
            switch (Mobicq.mobicq.getOptionsRef().getDisplayAdvertisementMode())
            {
              case 0:
                appendAdvertisement = true;
                ContactListContactItem.this.outgoingPlainMessagesCnt++;
                break;
              case 1:
                appendAdvertisement = (ContactListContactItem.this.outgoingPlainMessagesCnt == 0);
                ContactListContactItem.this.outgoingPlainMessagesCnt++;
                break;
            }

            // Construct plain message object and request new SendMessageAction
            PlainMessage plainMsg = new PlainMessage(Mobicq.mobicq.getIcqRef().getUin(), ContactListContactItem.this.getUin(), new Date(), MenuUtil.messageTextbox.getString(), appendAdvertisement);
            SendMessageAction sendMsgAct = new SendMessageAction(plainMsg);
            try
            {
              Mobicq.mobicq.getIcqRef().requestAction(sendMsgAct);
            }
            catch (MobicqException e)
            {
              MobicqException.handleException(e);
              if (e.isCritical()) return;
            }

            // Return to contact list
            Mobicq.mobicq.getContactListRef().update();
            Mobicq.mobicq.getContactListRef().activate();

          }
          // Send URL message (continue creation)
          else if (MenuUtil.menuList.getSelectedIndex() == 1)
          {

            // Reset and display textbox for entering URLs
            MenuUtil.urlTextbox.setString(null);
            MenuUtil.urlTextbox.setCommandListener(this);
            Mobicq.display.setCurrent(MenuUtil.urlTextbox);

          }

        }
        // URL has been entered
        else if (d == MenuUtil.urlTextbox)
        {

          // Abort if nothing has been entered
          if (MenuUtil.urlTextbox.getString().length() < 1)
          {
            this.activate();
          }

          // Construct URL message object and request new SendMessageAction
          UrlMessage urlMsg = new UrlMessage(Mobicq.mobicq.getIcqRef().getUin(), ContactListContactItem.this.getUin(), new Date(), MenuUtil.urlTextbox.getString(), MenuUtil.messageTextbox.getString());
          SendMessageAction sendMsgAct = new SendMessageAction(urlMsg);
          try
          {
            Mobicq.mobicq.getIcqRef().requestAction(sendMsgAct);
          }
          catch (MobicqException e)
          {
            MobicqException.handleException(e);
            if (e.isCritical()) return;
          }

          // Return to contact list
          Mobicq.mobicq.getContactListRef().update();
          Mobicq.mobicq.getContactListRef().activate();

        }

      }
      // Textbox has been canceled
      else if (c == MenuUtil.textboxCancelCommand)
      {
        this.activate();
      }

    }


    // Activates the contact item menu
    public void activate()
    {

      // Display message, if available
      if (ContactListContactItem.this.getNumOfAvailableMessages() > 0)
      {

        // Lock contact item object
        synchronized (ContactListContactItem.this)
        {

          // Get next message
          Message msg = (Message) ContactListContactItem.this.messages.elementAt(0);
          ContactListContactItem.this.messages.removeElementAt(0);

          // Construct form for displaying the next message
          Form msgDisplay = new Form(ResourceBundle.getString("org.mobicq.res.Text", "message_from") + " " + ContactListContactItem.this.getName());
          if (msg instanceof PlainMessage)
          {
            PlainMessage plainMsg = (PlainMessage) msg;
            msgDisplay.append(new ImageItem(null, ContactList.eventPlainMessageImg, ImageItem.LAYOUT_LEFT + ImageItem.LAYOUT_NEWLINE_BEFORE + ImageItem.LAYOUT_NEWLINE_AFTER, null));
            msgDisplay.append(new StringItem(null, plainMsg.getText()));
          }
          else if (msg instanceof UrlMessage)
          {
            UrlMessage urlMsg = (UrlMessage) msg;
            msgDisplay.append(new ImageItem(null, ContactList.eventUrlMessageImg, ImageItem.LAYOUT_LEFT + ImageItem.LAYOUT_NEWLINE_BEFORE + ImageItem.LAYOUT_NEWLINE_AFTER, null));
            msgDisplay.append(new StringItem(null, urlMsg.getUrl()));
            msgDisplay.append(new StringItem(null, urlMsg.getText()));
          }
          msgDisplay.addCommand(MenuUtil.msgCloseCommand);
          msgDisplay.addCommand(MenuUtil.msgReplyCommand);
          msgDisplay.setCommandListener(this);

          // Display message
          Mobicq.display.setCurrent(msgDisplay);

        }

      }
      // Display menu
      else
      {
        MenuUtil.menuList.setTitle(ContactListContactItem.this.name);
        MenuUtil.menuList.setSelectedIndex(0, true);
        MenuUtil.menuList.setCommandListener(this);
        Mobicq.display.setCurrent(MenuUtil.menuList);
      }

    }


  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  private static class MenuUtil
  {


    // Menu list
    private static List menuList;


    // Textbox for entering messages
    private static TextBox messageTextbox;


    // Textbox for entering URLs
    private static TextBox urlTextbox;


    // Abort command
    private static Command backCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "back"), Command.BACK, 1);


    // Message close command
    private static Command msgCloseCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "close"), Command.OK, 1);


    // Message close and reply command
    private static Command msgReplyCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "reply"), Command.OK, 1);


    // Textbox OK command
    private static Command textboxOkCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "ok"), Command.OK, 1);


    // Textbox Send command
    private static Command textboxSendCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "send"), Command.OK, 1);


    // Textbox cancel command
    private static Command textboxCancelCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "cancel"), Command.CANCEL, 2);


    // Initializer
    static
    {

      // Initialize the menu list
      MenuUtil.menuList = new List("set", Choice.IMPLICIT);
      MenuUtil.menuList.append(ResourceBundle.getString("org.mobicq.res.Text", "send_message"), null);
      MenuUtil.menuList.append(ResourceBundle.getString("org.mobicq.res.Text", "send_url"), null);
      MenuUtil.menuList.append(ResourceBundle.getString("org.mobicq.res.Text", "info"), null);
      MenuUtil.menuList.append(ResourceBundle.getString("org.mobicq.res.Text", "remove"), null);
      MenuUtil.menuList.addCommand(MenuUtil.backCommand);

      // Initialize the textbox for entering messages
      MenuUtil.messageTextbox = new TextBox(ResourceBundle.getString("org.mobicq.res.Text", "message"), null, 1000, TextField.ANY);
      MenuUtil.messageTextbox.addCommand(MenuUtil.textboxCancelCommand);

      // Initialize the textbox for entering URLs
      MenuUtil.urlTextbox = new TextBox(ResourceBundle.getString("org.mobicq.res.Text", "url"), null, 1000, TextField.URL);
      MenuUtil.urlTextbox.addCommand(MenuUtil.textboxCancelCommand);
      MenuUtil.urlTextbox.addCommand(MenuUtil.textboxSendCommand);

    }


  }


}
