/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/MobicqException.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq;


import org.mobicq.util.ResourceBundle;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;


public class MobicqException extends Exception
{


  // Returns the error description for the given error code
  public static String getErrDesc(int errCode, int extErrCode)
  {
    String errDesc = ResourceBundle.getString("org.mobicq.res.Text", "error_" + errCode);
    if (errDesc == null) errDesc = ResourceBundle.getString("ord.mobicq.res.Text", "error_100");
    int ext = errDesc.indexOf("EXT");
    return (errDesc.substring(0, ext) + extErrCode + errDesc.substring(ext + 3));
  }


  /****************************************************************************/


  // True, if this is a critial exception
  protected boolean critical;


  // True, if an error message should be presented to the user
  protected boolean displayMsg;


  // Constructs a critical MobicqException
  public MobicqException(int errCode, int extErrCode)
  {
    super(MobicqException.getErrDesc(errCode, extErrCode));
    this.critical = true;
    this.displayMsg = true;
  }


  // Constructs a non-critical MobicqException
  public MobicqException(int errCode, int extErrCode, boolean displayMsg)
  {
    super(MobicqException.getErrDesc(errCode, extErrCode));
    this.critical = false;
    this.displayMsg = displayMsg;
  }


  // Returns true if an error message should be presented to the user
  public boolean isDisplayMsg()
  {
    return (this.displayMsg);
  }


  // Returns true if this is a critical exception
  public boolean isCritical()
  {
    return (this.critical);
  }


  // Exception handler
  public synchronized static void handleException(MobicqException e)
  {

    // Critical exception
    if (e.isCritical())
    {

      // Reset comm. subsystem
      Mobicq.mobicq.getIcqRef().reset();

      // Display error message
      Alert errorMsg = new Alert(ResourceBundle.getString("org.mobicq.res.Text", "error"), e.getMessage(), null, AlertType.ERROR);
      errorMsg.setTimeout(Alert.FOREVER);
      Mobicq.mobicq.getMainMenuRef().activate(errorMsg);

    }
    // Non-critical exception
    else
    {

      // Display error message, if required
      if (e.isDisplayMsg())
      {
        Alert errorMsg = new Alert(ResourceBundle.getString("org.mobicq.res.Text", "warning"), e.getMessage(), null, AlertType.WARNING);
        errorMsg.setTimeout(Alert.FOREVER);
        Mobicq.display.setCurrent(errorMsg, Mobicq.display.getCurrent());
      }

    }

  }


}
