/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/Options.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer / Andreas Rossbacher
 *******************************************************************************/


package org.mobicq;


import org.mobicq.comm.Util;
import org.mobicq.util.ResourceBundle;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.lang.Byte;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.TextField;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotFoundException;


public class Options
{


  // Default values
  public static final String DEFAULT_SRV_HOST = "login.icq.com";
  public static final String DEFAULT_SRV_PORT = "5190";
  #sijapp target.if SIEMENS#
  public static final int DEFAULT_MSG_NOTIFICATION = 0;
  #sijapp target.end#
  public static final boolean DEFAULT_KEEP_CONN_ALIVE = true;
  public static final boolean DEFAULT_DISPLAY_DATE = false;
  public static final int DEFAULT_DISPLAY_ADVERTISEMENT = 0;
  public static final byte COST_PER_PACKET = 0;
  public static final byte COST_PER_DAY = 0;
  public static final long COST_PACKET_LENGTH = 1024;
  public static final String CURRENCY = "";
  public static final long DEFAULT_ONLINE_STATUS = ContactList.STATUS_ONLINE;


  /****************************************************************************/


  // UIN
  private String uin;


  // Password
  private String password;


  // Hostname of the login server
  private String srvHost;


  // Port of the login server
  private String srvPort;


  #sijapp target.if SIEMENS#
  // Message notification mode
  private int msgNotificationMode;
  #sijapp target.end#


  // Keep connection alive flag
  private boolean keepConnAlive;


  // Flag indicating we want to display the Mobicq advertisement
  private int displayAdvertisementMode;

  // Display date on splash screen
  private boolean displayDate;

  // Online status
  private long onlineStatus;

  //Cost per packet
  private byte costPerPacket;

  //Cost per day
  private byte costPerDay;

  //Length of packet to be charged in bytes
  private long costPacketLength;

  //Name or Symbol of currency
  private String currency;

  // Options form
  public OptionsForm optionsForm;


  // Constructor
  public Options()
  {

    // Try to load option values from record store
    try
    {
      this.load();
    }
    catch (Exception e)
    {
      this.setUin("");
      this.setPassword("");
      this.setSrvHost(Options.DEFAULT_SRV_HOST);
      this.setSrvPort(Options.DEFAULT_SRV_PORT);
      #sijapp target.if SIEMENS#
      this.setMsgNotificationMode(Options.DEFAULT_MSG_NOTIFICATION);
      #sijapp target.end#
      this.setDisplayDate(Options.DEFAULT_DISPLAY_DATE);
      this.setKeepConnAlive(Options.DEFAULT_KEEP_CONN_ALIVE);
      this.setDisplayAdvertisementMode(Options.DEFAULT_DISPLAY_ADVERTISEMENT);
      this.setCostPerPacket(Options.COST_PER_PACKET);
      this.setCostPerDay(Options.COST_PER_DAY);
      this.setCostPacketLength(Options.COST_PACKET_LENGTH);
      this.setCurrency(Options.CURRENCY);
      this.setOnlineStatus(Options.DEFAULT_ONLINE_STATUS);
    }

    // Construct option form
    this.optionsForm = new OptionsForm();
  }


  // Load option values from record store
  public void load() throws IOException, RecordStoreException
  {

    // Open record store
    RecordStore account = RecordStore.openRecordStore("account", false);

    // Temporary variables
    byte[] buf;
    ByteArrayInputStream bais;
    DataInputStream dis;

    // Get version info from record store
    buf = account.getRecord(1);
    bais = new ByteArrayInputStream(buf);
    dis = new DataInputStream(bais);
    if (!(dis.readUTF().equals(Mobicq.VERSION))) throw (new IOException());

    // Get UIN, server hostname and port, message notification flag, keep connection alive flag, advertisement flag, date display and status from record store
    buf = account.getRecord(2);
    bais = new ByteArrayInputStream(buf);
    dis = new DataInputStream(bais);
    this.setUin(dis.readUTF());
    this.setSrvHost(dis.readUTF());
    this.setSrvPort(dis.readUTF());
    #sijapp target.if SIEMENS#
    this.setMsgNotificationMode(dis.readInt());
    #sijapp target.else#
    dis.readInt();
    #sijapp target.end#
    this.setKeepConnAlive(dis.readBoolean());
    this.setDisplayDate(dis.readBoolean());
    this.setDisplayAdvertisementMode(dis.readInt());
    this.setOnlineStatus(dis.readLong());
    this.setCostPerPacket(dis.readByte());
    this.setCostPerDay(dis.readByte());
    this.setCostPacketLength(dis.readLong());
    this.setCurrency(dis.readUTF());

    // Get password from record store
    buf = account.getRecord(3);
    buf = Util.decipherPassword(buf);
    bais = new ByteArrayInputStream(buf);
    dis = new DataInputStream(bais);
    this.setPassword(dis.readUTF());

    // Close record store
    account.closeRecordStore();
  }


  // Save option values to record store
  public void save() throws IOException, RecordStoreException
  {

    // Open record store
    RecordStore account = RecordStore.openRecordStore("account", true);

    // Add empty records if necessary
    while (account.getNumRecords() < 4)
    {
      account.addRecord(null, 0, 0);
    }

    // Temporary variables
    byte[] buf;
    ByteArrayOutputStream baos;
    DataOutputStream dos;

    // Add version info to record store
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);
    dos.writeUTF(Mobicq.VERSION);
    buf = baos.toByteArray();
    account.setRecord(1, buf, 0, buf.length);

    // Add UIN, server hostname and port, message notification flag, keep connection alive flag, advertisement flag and status to the record store
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);
    dos.writeUTF(this.getUin());
    dos.writeUTF(this.getSrvHost());
    dos.writeUTF(this.getSrvPort());
    #sijapp target.if SIEMENS#
    dos.writeInt(this.getMsgNotificationMode());
    #sijapp target.else#
    dos.writeInt(0);
    #sijapp target.end#
    dos.writeBoolean(this.isKeepConnAlive());
    dos.writeBoolean(this.isDisplayDate());
    dos.writeInt(this.getDisplayAdvertisementMode());
    dos.writeLong(this.getOnlineStatus());
    dos.writeByte(this.getCostPerPacket());
    dos.writeByte(this.getCostPerDay());
    dos.writeLong(this.getCostPacketLength());
    dos.writeUTF(this.getCurrency());
    buf = baos.toByteArray();
    account.setRecord(2, buf, 0, buf.length);

    // Add the password to the record store
    baos = new ByteArrayOutputStream();
    dos = new DataOutputStream(baos);
    dos.writeUTF(this.getPassword());
    buf = baos.toByteArray();
    buf = Util.decipherPassword(buf);
    account.setRecord(3, buf, 0, buf.length);

    // Close record store
    account.closeRecordStore();

  }


  // Return UIN
  public synchronized String getUin()
  {
    return (new String(this.uin));
  }


  // Set UIN
  public synchronized void setUin(String uin)
  {
    this.uin = new String(uin);
  }


  // Return password
  public synchronized String getPassword()
  {
    return (new String(this.password));
  }


  // Set password
  public synchronized void setPassword(String password)
  {
    this.password = new String(password);
  }


  // Returns the hostname of the login server
  public synchronized String getSrvHost()
  {
    return (new String(this.srvHost));
  }


  // Sets the hostname of the login server
  public synchronized void setSrvHost(String srvHost)
  {
    this.srvHost = new String(srvHost);
  }


  // Returns the port of the login server
  public synchronized String getSrvPort()
  {
    return (new String(this.srvPort));
  }


  // Sets the port of the login server
  public synchronized void setSrvPort(String srvPort)
  {
    this.srvPort = new String(srvPort);
  }


  #sijapp target.if SIEMENS#
  // Returns the message notification option value
  public synchronized int getMsgNotificationMode()
  {
    return (this.msgNotificationMode);
  }
  #sijapp target.end#


    #sijapp target.if SIEMENS#
  // Sets the message notification option value
  public synchronized void setMsgNotificationMode(int msgNotificationMode)
  {
    this.msgNotificationMode = msgNotificationMode;
  }
  #sijapp target.end#


  // Returns the keep connection alive flag
  public synchronized boolean isKeepConnAlive()
  {
    return (this.keepConnAlive);
  }


  // Sets the keep connection alive flag
  public synchronized void setKeepConnAlive(boolean keepConnAlive)
  {
    this.keepConnAlive = keepConnAlive;
  }


  // Returns the display date
  public synchronized boolean isDisplayDate()
  {
    return (this.displayDate);
  }


  // Sets the display date flag
  public synchronized void setDisplayDate(boolean displayDate)
  {
    this.displayDate = displayDate;
  }


  // Returns the advertisement flag
  public synchronized int getDisplayAdvertisementMode()
  {
    return (this.displayAdvertisementMode);
  }


  // Set sthe advertisement flag
  public synchronized void setDisplayAdvertisementMode(int displayAdvertisementMode)
  {
    this.displayAdvertisementMode = displayAdvertisementMode;
  }


  // Returns the status
  public synchronized long getOnlineStatus()
  {
    return (this.onlineStatus);
  }


  // Sets the status
  public synchronized void setOnlineStatus(long onlineStatus)
  {
    this.onlineStatus = onlineStatus;
  }

  // Sets the Cost per packet
  public synchronized void setCostPerPacket(byte amount)
  {
    this.costPerPacket = amount;
  }

  // Sets the Cost per day
  public synchronized void setCostPerDay(byte amount)
  {
    this.costPerDay = amount;
  }

  // Sets the packet length in kilobytes
  public synchronized void setCostPacketLength(long length)
  {
    this.costPacketLength = length;
  }

  // Sets the currency string
  public synchronized void setCurrency(String currency)
  {
    this.currency = currency;
  }

  // Gets the Cost per packet
  public synchronized byte getCostPerPacket()
  {
    return (this.costPerPacket);
  }

  // Gets the Cost per day
  public synchronized byte getCostPerDay()
  {
    return (this.costPerDay);
  }

  // Gets the packet length in kilobytes
  public synchronized long getCostPacketLength()
  {
    return (this.costPacketLength);
  }

  //Gets the currency string
  public synchronized String getCurrency()
  {
    return (this.currency);
  }

  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  // Form for editing option values
  public class OptionsForm implements CommandListener
  {


    // Form elements
    private Form optionsForm;
    private TextField uinTextField;
    private TextField passwordTextField;
    private TextField srvHostTextField;
    private TextField srvPortTextField;
    #sijapp target.if SIEMENS#
    private ChoiceGroup msgNotificationModeChoiceGroup;
    #sijapp target.end#
    private ChoiceGroup keepConnAliveChoiceGroup;
    private ChoiceGroup displayDateChoiceGroup;
    private ChoiceGroup displayAdvertisementModeChoiceGroup;
    private TextField cppTextField;
    private TextField cpdTextField;
    private TextField plenTextField;
    private TextField curTextField;
    private Command saveCommand;


    // Constructor
    public OptionsForm()
    {

      // Initialize command
      this.saveCommand = new Command(ResourceBundle.getString("org.mobicq.res.Text", "save"), Command.SCREEN, 1);

      // Initialize options form
      this.optionsForm = new Form(ResourceBundle.getString("org.mobicq.res.Text", "options"));
      this.optionsForm.addCommand(this.saveCommand);
      this.optionsForm.setCommandListener(this);

      // Add account section
      this.uinTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "uin"), Options.this.getUin(), 9, TextField.NUMERIC);
      this.passwordTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "password"), Options.this.getPassword(), 8, TextField.PASSWORD);

      // Add server section
      this.srvHostTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "server_host"), Options.this.getSrvHost(), 32, TextField.ANY);
      this.srvPortTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "server_port"), Options.this.getSrvPort(), 5, TextField.NUMERIC);

      // Add message notification mode section
      #sijapp target.if SIEMENS#
      this.msgNotificationModeChoiceGroup = new ChoiceGroup(ResourceBundle.getString("org.mobicq.res.Text", "message_notification"), Choice.EXCLUSIVE);
      this.msgNotificationModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "no"), null);
      this.msgNotificationModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "beep"), null);
      this.msgNotificationModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "vibration"), null);
      this.msgNotificationModeChoiceGroup.setSelectedIndex(Options.this.getMsgNotificationMode(), true);
      #sijapp target.end#

      // Add keep connection alive setting
      this.keepConnAliveChoiceGroup = new ChoiceGroup(ResourceBundle.getString("org.mobicq.res.Text", "keep_conn_alive"), Choice.EXCLUSIVE);
      this.keepConnAliveChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "yes"), null);
      this.keepConnAliveChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "no"), null);
      this.keepConnAliveChoiceGroup.setSelectedIndex(Options.this.isKeepConnAlive() ? 0 : 1, true);

      // Add display date setting
      this.displayDateChoiceGroup = new ChoiceGroup(ResourceBundle.getString("org.mobicq.res.Text", "display_date"), Choice.EXCLUSIVE);
      this.displayDateChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "yes"), null);
      this.displayDateChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "no"), null);
      this.displayDateChoiceGroup.setSelectedIndex(Options.this.isDisplayDate() ? 0 : 1, true);

      // Add display advertisement setting
      this.displayAdvertisementModeChoiceGroup = new ChoiceGroup(ResourceBundle.getString("org.mobicq.res.Text", "display_advertisement"), Choice.EXCLUSIVE);
      this.displayAdvertisementModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "yes"), null);
      this.displayAdvertisementModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "once_a_session"), null);
      this.displayAdvertisementModeChoiceGroup.append(ResourceBundle.getString("org.mobicq.res.Text", "no"), null);
      this.displayAdvertisementModeChoiceGroup.setSelectedIndex(Options.this.getDisplayAdvertisementMode(), true);

      // Add cost section

      this.cppTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "cpp"), String.valueOf(Options.this.getCostPerPacket()), 4, TextField.NUMERIC);
      this.cpdTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "cpd"), String.valueOf(Options.this.getCostPerDay()), 4, TextField.NUMERIC);
      this.plenTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "plength"), String.valueOf(Options.this.getCostPacketLength() / 1024), 4, TextField.NUMERIC);
      this.curTextField = new TextField(ResourceBundle.getString("org.mobicq.res.Text", "currency"), Options.this.getCurrency(), 4, TextField.ANY);

    }


    // Activate options form
    public void activate()
    {
      while (this.optionsForm.size() > 0) this.optionsForm.delete(0);
      this.optionsForm.append(this.uinTextField);
      this.optionsForm.append(this.passwordTextField);
      this.optionsForm.append(this.srvHostTextField);
      this.optionsForm.append(this.srvPortTextField);
      #sijapp target.if SIEMENS#
      this.optionsForm.append(this.msgNotificationModeChoiceGroup);
      #sijapp target.end#
      this.optionsForm.append(this.keepConnAliveChoiceGroup);
      this.optionsForm.append(this.displayDateChoiceGroup);
      this.optionsForm.append(this.displayAdvertisementModeChoiceGroup);
      this.optionsForm.append(this.cppTextField);
      this.optionsForm.append(this.cpdTextField);
      this.optionsForm.append(this.plenTextField);
      this.optionsForm.append(this.curTextField);
      Mobicq.display.setCurrent(this.optionsForm);
    }


    // Command listener
    public void commandAction(Command c, Displayable d)
    {

      // Look for save command
      if (c == this.saveCommand)
      {
        try
        {

          // Get text field values and save to record store
          Options.this.setUin(this.uinTextField.getString());
          Options.this.setPassword(this.passwordTextField.getString());
          Options.this.setSrvHost(this.srvHostTextField.getString());
          Options.this.setSrvPort(this.srvPortTextField.getString());
          #sijapp target.if SIEMENS#
          Options.this.setMsgNotificationMode(this.msgNotificationModeChoiceGroup.getSelectedIndex());
          #sijapp target.end#
          Options.this.setKeepConnAlive(this.keepConnAliveChoiceGroup.isSelected(0));
          Options.this.setDisplayDate(this.displayDateChoiceGroup.isSelected(0));
          Options.this.setDisplayAdvertisementMode(this.displayAdvertisementModeChoiceGroup.getSelectedIndex());

          Options.this.setCostPerPacket(Byte.parseByte(this.cppTextField.getString()));
          Options.this.setCostPerDay(Byte.parseByte(this.cpdTextField.getString()));
          Options.this.setCostPacketLength(Byte.parseByte(this.plenTextField.getString()) * 1024);
          Options.this.setCurrency(this.curTextField.getString());
          Options.this.save();

        }
        catch (Exception e)
        {
          // Do nothing
        }

        // Active main menu/contact list
        if (Mobicq.mobicq.getIcqRef().isConnected())
        {
          Alert shouldReconnect = new Alert(ResourceBundle.getString("org.mobicq.res.Text", "notice"), ResourceBundle.getString("org.mobicq.res.Text", "options_effect"), null, AlertType.INFO);
          Mobicq.mobicq.getContactListRef().activate(shouldReconnect);
        }
        else
        {
          Mobicq.mobicq.getMainMenuRef().activate();
        }

      }

    }


  }


}
