/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/SplashCanvas.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer / Andreas Rossbacher
 *******************************************************************************/


package org.mobicq;


import org.mobicq.comm.ConnectAction;
import org.mobicq.comm.DisconnectAction;
import org.mobicq.comm.RequestInfoAction;
import org.mobicq.comm.UpdateContactListAction;
import org.mobicq.util.ResourceBundle;

import java.io.IOException;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.midlet.MIDletStateChangeException;
import java.util.Date;
import java.util.Calendar;
import java.util.TimerTask;


public class SplashCanvas extends Canvas
{


  // Location of the splash image (inside the JAR file)
  private static final String SPLASH_IMG = "/splash.png";


  // Image object, holds the splash image
  private static Image splash;


  // Font (and font height in pixels) used to display informational messages
  private static Font font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_ITALIC, Font.SIZE_SMALL);
  private static int height = font.getHeight();


  // Font used to display the logo (if image is not available)
  private static Font logoFont = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_LARGE);


  // Initializer block
  static
  {

    // Construct splash image
    try
    {
      SplashCanvas.splash = Image.createImage(SplashCanvas.SPLASH_IMG);
    }
    catch (IOException e)
    {
      // Do nothing
    }

  }


  /****************************************************************************/


  // Message to display beneath the splash image
  private String message;


  // Progress in percent
  private int progress;


  // True if keylock has been enabled
  private boolean isLocked; // = false


  // True if at least one message is available
  private boolean isMessageAvailable; // = false


  //True if date should be displayed
  private boolean displayDate = false;

  // Timestamp
  private Date pressed; // = null

  //Used for time to be displayed on the spash screen

  private Calendar time = Calendar.getInstance();

  // Constructor
  public SplashCanvas(String message)
  {
    this.message = new String(message);
    this.progress = 0;
  }


  // Constructor, blank message
  public SplashCanvas()
  {
    this("");
  }


  // Returns the informational message
  public synchronized String getMessage()
  {
    return (this.message);
  }


  // Sets the informational message
  public synchronized void setMessage(String message)
  {
    this.message = new String(message);
    this.progress = 0;
  }


  // Returns the current progress in percent
  public synchronized int getProgress()
  {
    return (this.progress);
  }


  // Sets the current progress in percent (and request screen refresh)
  public synchronized void setProgress(int progress)
  {
    this.progress = progress;
    this.repaint(0, this.getHeight() - SplashCanvas.height - 2, this.getWidth(), SplashCanvas.height + 2);
  }


  // Enable keylock
  public synchronized void lock()
  {
    this.setMessage(ResourceBundle.getString("org.mobicq.res.Text", "keylock_enabled"));
    this.setProgress(0);
    this.isLocked = true;
    Mobicq.display.setCurrent(this);
  }


  // Disable keylock
  private synchronized void unlock()
  {
    this.isLocked = false;
    this.isMessageAvailable = false;
    Mobicq.mobicq.getContactListRef().activate();
  }


  // Called when message has been received
  public synchronized void messageAvailable()
  {
    if (this.isLocked)
    {
      this.isMessageAvailable = true;
      this.repaint();
    }
  }


  // Called when a key is pressed
  protected void keyPressed(int keyCode)
  {
    if (this.isLocked && (keyCode == Canvas.KEY_POUND))
    {
      this.pressed = new Date();
    }
  }


  // Called when a key is repeated (held down)
  protected void keyRepeated(int keyCode)
  {
    if (this.isLocked && (keyCode == Canvas.KEY_POUND))
    {
      if ((this.pressed.getTime() + 1000) < System.currentTimeMillis())
      {
        this.unlock();
      }
    }
  }


  // Called when a key is released
  protected void keyReleased(int keyCode)
  {
    if (this.isLocked && (keyCode == Canvas.KEY_POUND))
    {
      if ((this.pressed.getTime() + 1000) < System.currentTimeMillis())
      {
        this.unlock();
      }
    }
  }

  // Called to get a date String
  protected String getDateString()
  {
    time.setTime(new Date());
    String datestr = new String("failed");
    datestr = this.makeTwo(time.get(Calendar.DAY_OF_MONTH)) + "." +
        this.makeTwo(time.get(Calendar.MONTH) + 1) + "." +
        String.valueOf(time.get(Calendar.YEAR)) + " " +
        this.makeTwo(time.get(Calendar.HOUR_OF_DAY)) + ":" +
        this.makeTwo(time.get(Calendar.MINUTE));
    return datestr;
  }

  //Date print utility
  protected String makeTwo(int number)
  {
    if (number < 10)
      return ("0" + String.valueOf(number));
    else
      return (String.valueOf(number));
  }

  // Render the splash image
  protected void paint(Graphics g)
  {

    // Do we need to draw the splash image?
    if (g.getClipY() < this.getHeight() - SplashCanvas.height - 2)
    {

      // Clear background
      g.setColor(255, 255, 255);
      g.fillRect(0, 0, this.getWidth(), this.getHeight());

      // Display splash image (or text)
      if (SplashCanvas.splash != null)
      {
        g.drawImage(SplashCanvas.splash, this.getWidth() / 2, this.getHeight() / 2, Graphics.HCENTER | Graphics.VCENTER);

      }
      else
      {
        g.setColor(0, 0, 0);
        g.setFont(SplashCanvas.logoFont);
        g.drawString("mob icq", this.getWidth() / 2, this.getHeight() / 2 + 5, Graphics.HCENTER | Graphics.BASELINE);
        g.setFont(SplashCanvas.font);
      }

      // Display message icon, if keylock is enabled
      if (this.isLocked && this.isMessageAvailable)
      {
        g.drawImage(ContactList.eventPlainMessageImg, 1, 1, Graphics.LEFT | Graphics.TOP);


      }

      // Display the time and date
      try
      {
        displayDate = Mobicq.mobicq.getOptionsRef().isDisplayDate();
      }
      catch (NullPointerException e)
      { /* Do nothing */
      }
      if (this.displayDate)
      {
        g.setFont(SplashCanvas.font);
        g.drawString(this.getDateString(), this.getWidth() / 2, this.getHeight() / 2 - 10, Graphics.HCENTER | Graphics.BASELINE);
      }

    }

    // Draw white bottom bar
    g.setColor(255, 255, 255);
    g.fillRect(0, this.getHeight() - SplashCanvas.height - 2, this.getWidth(), SplashCanvas.height + 2);

    // Display message, if available
    if ((this.message != null) && (this.message.length() > 0))
    {

      // Draw message
      g.setColor(0, 0, 0);
      g.setFont(SplashCanvas.font);
      g.drawString(this.message, this.getWidth() / 2, this.getHeight(), Graphics.BOTTOM | Graphics.HCENTER);

      // Draw current progress
      int progressPx = this.getWidth() / 100 * this.progress;
      g.setClip(0, this.getHeight() - SplashCanvas.height - 2, progressPx, SplashCanvas.height + 2);
      g.setColor(0, 0, 191);
      g.fillRect(0, this.getHeight() - SplashCanvas.height - 2, progressPx, SplashCanvas.height + 2);
      g.setColor(255, 255, 255);
      g.drawString(this.message, this.getWidth() / 2, this.getHeight(), Graphics.BOTTOM | Graphics.HCENTER);

    }

  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  // Activates the contact list after connection has been established
  static class ConnectTimerTask extends TimerTask
  {


    // Reference to ConnectAction
    private ConnectAction connectAct;


    // Constructor
    public ConnectTimerTask(ConnectAction connectAct)
    {
      this.connectAct = connectAct;
    }


    // Timer routine
    public void run()
    {
      Mobicq.mobicq.getSplashCanvasRef().setProgress(this.connectAct.getProgress());
      if (this.connectAct.isCompleted())
      {
        Mobicq.mobicq.getContactListRef().resetListPosition();
        Mobicq.mobicq.getContactListRef().activate();
        this.cancel();
      }
      else if (this.connectAct.isError())
      {
        this.cancel();
      }
    }


  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  // Activates the main menu after connection has been terminated
  static class DisconnectTimerTask extends TimerTask
  {


    // Reference to DisconnectAction
    private DisconnectAction disconnectAct;


    // Exit after disconnecting?
    private boolean exit;


    // Constructor
    public DisconnectTimerTask(DisconnectAction disconnectAct, boolean exit)
    {
      this.disconnectAct = disconnectAct;
      this.exit = exit;
    }


    // Timer routine
    public void run()
    {
      if (this.disconnectAct.isCompleted())
      {
        this.cancel();
        if (this.exit)
        {
          try
          {
            Mobicq.mobicq.destroyApp(true);
          }
          catch (MIDletStateChangeException e)
          { /* Do nothing */
          }
        }
        else
        {
          Mobicq.mobicq.getMainMenuRef().activate();
        }
      }
      else if (this.disconnectAct.isError())
      {
        this.cancel();
      }
    }


  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  // Waits until meta information is available
  static class RequestInfoTimerTask extends TimerTask
  {


    // Reference to RequestInfoAction
    private RequestInfoAction requestInfoAct;


    // Constructor
    public RequestInfoTimerTask(RequestInfoAction requestInfoAct)
    {
      this.requestInfoAct = requestInfoAct;
    }


    // Timer routine
    public void run()
    {
      if (this.requestInfoAct.isCompleted())
      {
        String uin = this.requestInfoAct.getUin();
        String nick = this.requestInfoAct.getNick();
        String name = (this.requestInfoAct.getFirstName() + " " + this.requestInfoAct.getLastName()).trim();
        String email = this.requestInfoAct.getEmail();
        String infoText = ResourceBundle.getString("org.mobicq.res.Text", "uin") + ": " + (uin.length() > 0 ? uin : "-") + (char) (0x0D) +
            ResourceBundle.getString("org.mobicq.res.Text", "nick") + ": " + (nick.length() > 0 ? nick : "-") + (char) (0x0D) +
            ResourceBundle.getString("org.mobicq.res.Text", "name") + ": " + (name.length() > 0 ? name : "-") + (char) (0x0D) +
            ResourceBundle.getString("org.mobicq.res.Text", "email") + ": " + (email.length() > 0 ? email : "-");
        Alert info = new Alert(ResourceBundle.getString("org.mobicq.res.Text", "info"), infoText, null, AlertType.INFO);
        Mobicq.mobicq.getContactListRef().activate(info);
        this.cancel();
      }
      else if (this.requestInfoAct.isError())
      {
        Mobicq.mobicq.getContactListRef().activate();
        MobicqException.handleException(new MobicqException(161, 0, true));
        this.cancel();
      }
    }


  }


  /****************************************************************************/
  /****************************************************************************/
  /****************************************************************************/


  // Waits until contact listupdate is completed
  static class UpdateContactListTimerTask extends TimerTask
  {


    // Reference to UpdateContactListAction
    private UpdateContactListAction updateContactListAct;


    // Constructor
    public UpdateContactListTimerTask(UpdateContactListAction updateContactListAct)
    {
      this.updateContactListAct = updateContactListAct;
    }


    // Timer routine
    public void run()
    {
      if (this.updateContactListAct.isCompleted())
      {
        Mobicq.mobicq.getContactListRef().update();
        Mobicq.mobicq.getContactListRef().activate();
        this.cancel();
      }
      else if (this.updateContactListAct.isError())
      {
        Mobicq.mobicq.getContactListRef().update();
        Mobicq.mobicq.getContactListRef().activate();
        MobicqException.handleException(new MobicqException(154, 2, true));
        this.cancel();
      }
    }


  }


}
