/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/comm/ActionListener.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq.comm;


import org.mobicq.ContactList;
import org.mobicq.Mobicq;
import org.mobicq.MobicqException;
import java.util.Date;


public class ActionListener {


  // ICQ object
  protected Icq icq;


  // Set ICQ object
  protected void setIcq(Icq icq) {
    this.icq = icq;
  }


  // Forwards received packet
  protected void forward(Packet packet) throws MobicqException {

    // Watch out for channel 4 (Disconnect) packets
    if (packet instanceof DisconnectPacket)
    {
      DisconnectPacket disconnectPacket = (DisconnectPacket) packet;

      // Throw exception
      if (disconnectPacket.getError() == 0x0001)
      {   // Multiple logins
        throw (new MobicqException(110, 0));
      }
      else
      {   // Unknown error
        throw (new MobicqException(100, 0));
      }

    }


    /**************************************************************************/

    // Watch out for channel 2 (SNAC) packets
    if (packet instanceof SnacPacket)
    {
      SnacPacket snacPacket = (SnacPacket) packet;

      // Watch out for SRV_USERONLINE packets
      if ((snacPacket.getFamily() == SnacPacket.SRV_USERONLINE_FAMILY) && (snacPacket.getCommand() == SnacPacket.SRV_USERONLINE_COMMAND))
      {

        // Get data
        byte[] buf = snacPacket.getData();

        // Get UIN of the contact changing status
        int uinLen = Util.getByte(buf, 0);
        String uin = Util.byteArrayToString(buf, 1, uinLen);

        // Get new status
        long status = ContactList.STATUS_ONLINE;
        int marker = 1 + uinLen + 2;
        int tlvNum = Util.getWord(buf, marker);
        marker += 2;
        for (int i = 0; i < tlvNum; i++)
        {
          int tlvType = Util.getWord(buf, marker);
          byte[] tlvData = Util.getTlv(buf, marker);
          if (tlvType == 0x0006)
          {
            status = Util.getDWord(tlvData, 0);
          }
          marker += 2 + 2 + tlvData.length;
        }

        // Update contact list
        Mobicq.mobicq.getContactListRef().update(uin, status);

      }

      /************************************************************************/

      // Watch out for SRV_USEROFFLINE packets
      if ((snacPacket.getFamily() == SnacPacket.SRV_USEROFFLINE_FAMILY) && (snacPacket.getCommand() == SnacPacket.SRV_USEROFFLINE_COMMAND))
      {

        // Get raw data
        byte[] buf = snacPacket.getData();

        // Get UIN of the contact that goes offline
        int uinLen = Util.getByte(buf, 0);
        String uin = Util.byteArrayToString(buf, 1, uinLen);

        // Update contact list
        Mobicq.mobicq.getContactListRef().update(uin, ContactList.STATUS_OFFLINE);

      }

      /************************************************************************/

      // Watch out for SRV_RECVMSG
      if ((snacPacket.getFamily() == SnacPacket.SRV_RECVMSG_FAMILY) && (snacPacket.getCommand() == SnacPacket.SRV_RECVMSG_COMMAND))
      {

        // Get raw data, initialize marker
        byte[] buf = snacPacket.getData();
        int marker = 0;

        // Check length
        if (buf.length < 11)
        {
          throw (new MobicqException(150, 0, false));
        }

        // Get message format
        marker += 8;
        int format = Util.getWord(buf, marker);
        marker += 2;

        // Get UIN length
        int uinLen = Util.getByte(buf, marker);
        marker += 1;

        // Check length
        if (buf.length < marker + uinLen + 4)
        {
          throw (new MobicqException(150, 1, false));
        }

        // Get UIN
        String uin = Util.byteArrayToString(buf, marker, uinLen);
        marker += uinLen;

        // Skip WARNING
        marker += 2;

        // Skip TLVS
        int tlvCount = Util.getWord(buf, marker);
        marker += 2;
        for (int i = 0; i < tlvCount; i++)
        {
          byte[] tlvData = Util.getTlv(buf, marker);
          if (tlvData == null)
          {
            throw (new MobicqException(150, 2, false));
          }
          marker += 4 + tlvData.length;
        }

        // Get message data and initialize marker
        byte[] msgBuf;
        int tlvType;
        do
        {
          msgBuf = Util.getTlv(buf, marker);
          if (msgBuf == null)
          {
            throw (new MobicqException(150, 3, false));
          }
          tlvType = Util.getWord(buf, marker);
          marker += 4 + msgBuf.length;
        }
        while ((tlvType != 0x0002) && (tlvType != 0x0005));
        int msgMarker = 0;

        //////////////////////
        // Message format 1 //
        //////////////////////
        if (format == 0x0001)
        {

          // Variables for all possible TLVs
          byte[] capabilities = null;
          byte[] message = null;

          // Read all TLVs
          while (msgMarker < msgBuf.length)
          {

            // Get next TLV
            byte[] tlvValue = Util.getTlv(msgBuf, msgMarker);
            if (tlvValue == null)
            {
              throw (new MobicqException(151, 0, false));
            }

            // Get type of next TLV
            tlvType = Util.getWord(msgBuf, msgMarker);

            // Update markers
            msgMarker += 4 + tlvValue.length;

            // Save value
            switch (tlvType)
            {
              case 0x0501:   // capabilities
                capabilities = tlvValue;
                break;
              case 0x0101:   // message
                message = tlvValue;
                break;
              default:
                throw (new MobicqException(151, 1, false));
            }

          }

          // Process packet if at least the message TLV was present
          if (message != null)
          {

            // Check length of message
            if (message.length < 4)
            {
              throw (new MobicqException(151, 2, false));
            }

            // Get message text
            String text = Util.crlfToCr(Util.byteArrayToString(message, 4, message.length - 4));

            // Construct object which encapsulates the received plain message
            PlainMessage plainMsg = new PlainMessage(uin, this.icq.getUin(), new Date(), text);
            Mobicq.mobicq.getContactListRef().addMessage(plainMsg);

          }

        }
        //////////////////////
        // Message format 2 //
        //////////////////////
        else if (format == 0x0002)
        {

          // Check length
          if (msgBuf.length < 10)
          {
            throw (new MobicqException(152, 0, false));
          }

          // Get and validate SUB_MSG_TYPE2.ACKTYPE
          int ackType = Util.getWord(msgBuf, msgMarker);
          if (ackType != 0x0000) return;   // Only normal messages are supported yet
          msgMarker += 2;

          // Skip SUB_MSG_TYPE2.TIME and SUB_MSG_TYPE2.ID
          msgMarker += 4 + 4;

          // Check length
          if (msgBuf.length < msgMarker + 16)
          {
            throw (new MobicqException(152, 1, false));
          }

          // Skip SUB_MSG_TYPE2.CAPABILITY
          msgMarker += 16;

          // Get message data and initialize marker
          byte[] msg2Buf;
          do
          {
            msg2Buf = Util.getTlv(msgBuf, msgMarker);
            if (msg2Buf == null)
            {
              throw (new MobicqException(152, 2, false));
            }
            tlvType = Util.getWord(msgBuf, msgMarker);
            msgMarker += 4 + msg2Buf.length;
          }
          while (tlvType != 0x2711);
          int msg2Marker = 0;

          // Check length
          if (msg2Buf.length < 2 + 2 + 16 + 3 + 4 + 2 + 2 + 2 + 12 + 2 + 2 + 2 + 2)
          {
            throw (new MobicqException(152, 3, false));
          }

          // Skip values up to (and including) SUB_MSG_TYPE2.UNKNOWN (before MSGTYPE)
          msg2Marker += 2 + 2 + 16 + 3 + 4 + 2 + 2 + 2 + 12;

          // Get and validate message type
          int msgType = Util.getWord(msg2Buf, msg2Marker, false);
          msg2Marker += 2;
          if ((msgType != 0x0001) && (msgType != 0x0004) && (msgType != 0x001A)) return;

          // Skip UNKNOWN and PRIORITY
          msg2Marker += 2 + 2;

          // Get length of text
          int textLen = Util.getWord(msg2Buf, msg2Marker, false);
          msg2Marker += 2;

          // Check length
          if (msg2Buf.length < msg2Marker + textLen) {
            throw (new MobicqException(152, 4, false));
          }

          // Get text
          String text = Util.crlfToCr(Util.byteArrayToString(msg2Buf, msg2Marker, textLen));
          msg2Marker += textLen;

          // Plain message
          if ((msgType == 0x0001) && (text.length() > 0))
          {

            // Forward message to contact list
            PlainMessage message = new PlainMessage(uin, this.icq.getUin(), new Date(), text);
            Mobicq.mobicq.getContactListRef().addMessage(message);

            // Acknowledge message
            byte[] ackBuf = new byte[10 + 1 + uinLen + 2 + 51 + 3];
            int ackMarker = 0;
            System.arraycopy(buf, 0, ackBuf, ackMarker, 10);
            ackMarker += 10;
            Util.putByte(ackBuf, ackMarker, uinLen);
            ackMarker += 1;
            byte[] uinRaw = Util.stringToByteArray(uin);
            System.arraycopy(uinRaw, 0, ackBuf, ackMarker, uinRaw.length);
            ackMarker += uinRaw.length;
            Util.putWord(ackBuf, ackMarker, 0x0003);
            ackMarker += 2;
            System.arraycopy(msg2Buf, 0, ackBuf, ackMarker, 51);
            ackMarker += 51;
            Util.putWord(ackBuf, ackMarker, 0x0001, false);
            ackMarker += 2;
            Util.putByte(ackBuf, ackMarker, 0x00);
            ackMarker += 1;
            SnacPacket ackPacket = new SnacPacket(SnacPacket.CLI_ACKMSG_FAMILY, SnacPacket.CLI_ACKMSG_COMMAND, 0, new byte[0], ackBuf);
            this.icq.c.sendPacket(ackPacket);

          }
          // URL message
          else if (msgType == 0x0004)
          {

            // Search for delimiter
            int delim = text.indexOf(0xFE);

            // Split message, if delimiter could be found
            String urlText;
            String url;
            if (delim != -1)
            {
              urlText = text.substring(0, delim);
              url = text.substring(delim + 1);
            }
            else
            {
              urlText = text;
              url = "";
            }

            // Forward message message to contact list
            UrlMessage message = new UrlMessage(uin, this.icq.getUin(), new Date(), url, urlText);
            Mobicq.mobicq.getContactListRef().addMessage(message);

            // Acknowledge message
            byte[] ackBuf = new byte[10 + 1 + uinLen + 2 + 51 + 3];
            int ackMarker = 0;
            System.arraycopy(buf, 0, ackBuf, ackMarker, 10);
            ackMarker += 10;
            Util.putByte(ackBuf, ackMarker, uinLen);
            ackMarker += 1;
            byte[] uinRaw = Util.stringToByteArray(uin);
            System.arraycopy(uinRaw, 0, ackBuf, ackMarker, uinRaw.length);
            ackMarker += uinRaw.length;
            Util.putWord(ackBuf, ackMarker, 0x0003);
            ackMarker += 2;
            System.arraycopy(msg2Buf, 0, ackBuf, ackMarker, 51);
            ackMarker += 51;
            Util.putWord(ackBuf, ackMarker, 0x0001, false);
            ackMarker += 2;
            Util.putByte(ackBuf, ackMarker, 0x00);
            ackMarker += 1;
            SnacPacket ackPacket = new SnacPacket(SnacPacket.CLI_ACKMSG_FAMILY, SnacPacket.CLI_ACKMSG_COMMAND, 0, new byte[0], ackBuf);
            this.icq.c.sendPacket(ackPacket);

          }
          // Extended message
          else if (msgType == 0x001A)
          {

            // Check length
            if (msg2Buf.length < msg2Marker + 2 + 18 + 4)
            {
              throw (new MobicqException(152, 5, false));
            }

            // Save current marker position
            int extDataStart = msg2Marker;

            // Skip EXTMSG.LEN and EXTMSG.UNKNOWN
            msg2Marker += 2 + 18;

            // Get length of plugin string
            long pluginLen = Util.getDWord(msg2Buf, msg2Marker, false);
            msg2Marker += 4;

            // Check length
            if (msg2Buf.length < msg2Marker + pluginLen + 15 + 4 + 4)
            {
              throw (new MobicqException(152, 6, false));
            }

            // Get plugin string
            String plugin = Util.byteArrayToString(msg2Buf, msg2Marker, (int) pluginLen);
            msg2Marker += pluginLen;

            // Skip EXTMSG.UNKNOWN and EXTMSG.LEN
            msg2Marker += 15 + 4;

            // Get length of text
            textLen = (int) Util.getDWord(msg2Buf, msg2Marker, false);
            msg2Marker += 4;

            // Check length
            if (msg2Buf.length < msg2Marker + textLen)
            {
              throw (new MobicqException(152, 7, false));
            }

            // Get text
            text = Util.crlfToCr(Util.byteArrayToString(msg2Buf, msg2Marker, textLen));
            msg2Marker += textLen;

            // URL message
            if (plugin.equals("Send Web Page Address (URL)"))
            {

              // Search for delimiter
              int delim = text.indexOf(0xFE);

              // Split message, if delimiter could be found
              String urlText;
              String url;
              if (delim != -1)
              {
                urlText = text.substring(0, delim);
                url = text.substring(delim + 1);
              }
              else
              {
                urlText = text;
                url = "";
              }

              // Forward message message to contact list
              UrlMessage message = new UrlMessage(uin, this.icq.getUin(), new Date(), url, urlText);
              Mobicq.mobicq.getContactListRef().addMessage(message);

              // Acknowledge message
              byte[] ackBuf = new byte[10 + 1 + uinLen + 2 + 51 + 3 + 20 + 4 + (int) pluginLen + 19 + 4 + textLen];
              int ackMarker = 0;
              System.arraycopy(buf, 0, ackBuf, ackMarker, 10);
              ackMarker += 10;
              Util.putByte(ackBuf, ackMarker, uinLen);
              ackMarker += 1;
              byte[] uinRaw = Util.stringToByteArray(uin);
              System.arraycopy(uinRaw, 0, ackBuf, ackMarker, uinRaw.length);
              ackMarker += uinRaw.length;
              Util.putWord(ackBuf, ackMarker, 0x0003);
              ackMarker += 2;
              System.arraycopy(msgBuf, 0, ackBuf, ackMarker, 51);
              ackMarker += 51;
              Util.putWord(ackBuf, ackMarker, 0x0001, false);
              ackMarker += 2;
              Util.putByte(ackBuf, ackMarker, 0x00);
              ackMarker += 1;
              System.arraycopy(msg2Buf, extDataStart, ackBuf, ackMarker, 20 + 4 + (int) pluginLen + 19 + 4 + textLen);
              SnacPacket ackPacket = new SnacPacket(SnacPacket.CLI_ACKMSG_FAMILY, SnacPacket.CLI_ACKMSG_COMMAND, 0, new byte[0], ackBuf);
              this.icq.c.sendPacket(ackPacket);

            }
            // Other messages
            else
            {
              // Discard
            }

          }

        }
        //////////////////////
        // Message format 4 //
        //////////////////////
        else if (format == 0x0004)
        {

          // Check length
          if (msgBuf.length < 8)
          {
            throw (new MobicqException(153, 0, false));
          }

          // Skip SUB_MSG_TYPE4.UIN
          msgMarker += 4;

          // Get SUB_MSG_TYPE4.MSGTYPE
          int msgType = Util.getWord(msgBuf, msgMarker, false);
          msgMarker += 2;

          // Only plain messages and URL messagesa are supported
          if ((msgType != 0x0001) && (msgType != 0x0004)) return;

          // Get length of text
          int textLen = Util.getWord(msgBuf, msgMarker, false);
          msgMarker += 2;

          // Check length (exact match required)
          if (msgBuf.length != 8 + textLen)
          {
            throw (new MobicqException(153, 1, false));
          }

          // Get text
          String text = Util.crlfToCr(Util.byteArrayToString(msgBuf, msgMarker, textLen));
          msgMarker += textLen;

          // Plain message
          if (msgType == 0x0001)
          {

            // Forward message to contact list
            PlainMessage plainMsg = new PlainMessage(uin, this.icq.getUin(), new Date(), text);
            Mobicq.mobicq.getContactListRef().addMessage(plainMsg);

          }
          // URL message
          else if (msgType == 0x0004)
          {

            // Search for delimiter
            int delim = text.indexOf(0xFE);

            // Split message, if delimiter could be found
            String urlText;
            String url;
            if (delim != -1)
            {
              urlText = text.substring(0, delim);
              url = text.substring(delim + 1);
            }
            else
            {
              urlText = text;
              url = "";
            }

            // Forward message message to contact list
            UrlMessage urlMsg = new UrlMessage(uin, this.icq.getUin(), new Date(), url, urlText);
            Mobicq.mobicq.getContactListRef().addMessage(urlMsg);

          }

        }

      }

    }

  }


}
