/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/comm/Packet.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq.comm;


import org.mobicq.MobicqException;


abstract class Packet
{


  // Channel constants
  public static final int CHANNEL_CONNECT = 0x01;
  public static final int CHANNEL_SNAC = 0x02;
  public static final int CHANNEL_ERROR = 0x03;
  public static final int CHANNEL_DISCONNECT = 0x04;
  public static final int CHANNEL_PING = 0x05;


  // FLAP sequence number
  protected int sequence;


  // Returns the FLAP sequence number
  public int getSequence()
  {
    return (this.sequence);
  }


  // Sets the FLAP sequence number
  void setSequence(int sequence)
  {
    this.sequence = sequence;
  }


  // Returns the package as byte array
  public abstract byte[] toByteArray();


  // Parses given byte array and returns a Packet object
  public static Packet parse(byte[] buf, int off, int len) throws MobicqException
  {

    // Check length (min. 6 bytes)
    if (len < 6)
    {
      throw (new MobicqException(130, 0));
    }

    // Verify FLAP.ID
    if (Util.getByte(buf, off) != 0x2A)
    {
      throw (new MobicqException(130, 1));
    }

    // Get and verify FLAP.CHANNEL
    int channel = Util.getByte(buf, off + 1);
    if ((channel < 1) || (channel > 5))
    {
      throw (new MobicqException(130, 2));
    }

    // Verify FLAP.LENGTH
    int length = Util.getWord(buf, off + 4);
    if ((length + 6) != len)
    {
      throw (new MobicqException(130, 3));
    }

    // Parsing is done by a subclass
    switch (channel)
    {
      case Packet.CHANNEL_CONNECT:
        return (ConnectPacket.parse(buf, off, len));
      case Packet.CHANNEL_SNAC:
        return (SnacPacket.parse(buf, off, len));
      case Packet.CHANNEL_ERROR:
        return (ErrorPacket.parse(buf, off, len));
      case Packet.CHANNEL_DISCONNECT:
        return (DisconnectPacket.parse(buf, off, len));
      case Packet.CHANNEL_PING:
        return (PingPacket.parse(buf, off, len));
      default:
        throw (new MobicqException(131, 0));
    }

  }


  // Parses given byte array and returns a Packet object
  public static Packet parse(byte[] buf) throws MobicqException
  {
    return (Packet.parse(buf, 0, buf.length));
  }


}
