/*******************************************************************************
 Mobicq - J2ME ICQ client
 Copyright (C) 2003  Manuel Linsmayer

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************
 File: src/org/mobicq/comm/SetStatusAction.java
 Version: 0.1b5  Date: 2003/08/25
 Author: Manuel Linsmayer
 *******************************************************************************/


package org.mobicq.comm;


import org.mobicq.Mobicq;
import org.mobicq.MobicqException;


public class SetOnlineStatusAction extends Action
{


  // CLI_SETSTATUS packet data
  public static final byte[] CLI_SETSTATUS_DATA = ConnectAction.CLI_SETSTATUS_DATA;


  /****************************************************************************/


  // Requested online status
  private long onlineStatus;


  // Constructor
  public SetOnlineStatusAction(long onlineStatus)
  {
    this.onlineStatus = onlineStatus;
  }


  // Returns true if the action can be performed
  public boolean isExecutable()
  {
    return (this.icq.isConnected());
  }


  // Returns true if this is an exclusive command
  public boolean isExclusive()
  {
    return (false);
  }


  // Init action
  protected void init() throws MobicqException
  {

    // Convert online status
    long onlineStatus = Util.translateStatusSend(this.onlineStatus);

    // Send a CLI_SETSTATUS packet
    Util.putDWord(SetOnlineStatusAction.CLI_SETSTATUS_DATA, 4, onlineStatus);
    SnacPacket packet = new SnacPacket(SnacPacket.CLI_SETSTATUS_FAMILY,
                       SnacPacket.CLI_SETSTATUS_COMMAND,
                       0x00000000,
                       new byte[0],
                       SetOnlineStatusAction.CLI_SETSTATUS_DATA);
    this.icq.c.sendPacket(packet);

    // Save new online status
    Mobicq.mobicq.getOptionsRef().setOnlineStatus(this.onlineStatus);
    try
    {
      Mobicq.mobicq.getOptionsRef().save();
    }
    catch (Exception e)
    {
      // Do nothing
    }

  }


  // Forwards received packet, returns true if packet was consumed
  protected boolean forward(Packet packet) throws MobicqException
  {
    return (false);
  }


  // Returns true if the action is completed
  public boolean isCompleted()
  {
    return (true);
  }


  // Returns true if an error has occured
  public boolean isError()
  {
    return (false);
  }


}
