/*******************************************************************************
Mobicq - J2ME ICQ client
Copyright (C) 2003  Manuel Linsmayer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
********************************************************************************
File: src/org/mobicq/util/ResourceBundle.java
Version: 0.1b5  Date: 2003/08/25
Author: Manuel Linsmayer
*******************************************************************************/

package org.mobicq.util;

import java.util.Hashtable;

public class ResourceBundle {

  private static Hashtable groups = new Hashtable();

  public static synchronized Object getObject(String group, String key) {
    ResourceBundle bundle;

    synchronized (groups) {
      bundle = (ResourceBundle) groups.get(group);
      if (bundle == null) {
        bundle = loadBundle(group);
      }
    }

    return bundle.getResource(key);
  }

  public static synchronized String getString(String group, String key) {
    return (String) getObject(group, key);
  }

  public static ResourceBundle loadBundle(String name) {
    ResourceBundle bundle = null;
    Locale locale = Locale.getDefaultLocale();
    String language = locale.getLanguage();
    String country = locale.getCountry();

    try {
      bundle = (ResourceBundle) Class.forName(name).newInstance();
    }
    catch (Exception e) {
    }

    if (language != null) {
      ResourceBundle child;

      try {
        child = (ResourceBundle) Class.forName(name + "_" + language).newInstance();
        child.setParent(bundle);
        bundle = child;
      }
      catch (Exception e) {
      }

      if (country != null) {
        try {
          child = (ResourceBundle) Class.forName(name + "_" + language + "_" + country).newInstance();
          child.setParent(bundle);
          bundle = child;
        }
        catch (Exception e) {
        }
      }
    }

    if (bundle == null) {
      bundle = new ResourceBundle();
    }

    groups.put(name, bundle);
    return bundle;
  }

  protected Hashtable resources = new Hashtable();
  private ResourceBundle parent;

  protected ResourceBundle() {
  }

  protected void setParent(ResourceBundle parent) {
    this.parent = parent;
  }

  protected Object getResource(String key) {
    Object obj = null;

    if (resources != null) {
      obj = resources.get(key);
    }

    if ((obj == null) && (parent != null)){
      obj = parent.getResource(key);
    }

    return obj;
  }

  public static class Locale {

    private String language = "en";
    private String country = "US";

    public Locale(String language, String country) {
      this.language = language;
      this.country = country;
    }

    public Locale(String locale) {
      if ((locale != null) && (locale.length() > 0)) {
        int pos = locale.indexOf("_");
        if (pos == -1) {
          this.language = new String(locale);
        }
        else {
          this.language = locale.substring(0, pos);
          locale = locale.substring(pos+1);
          pos = locale.indexOf("_");
          if (pos == -1) {
            this.country = new String(locale);
          }
          else {
            this.country = locale.substring(0, pos);
          }
        }
      }
    }

    public String getLanguage() {
      return (this.language);
    }

    public String getCountry() {
      return (this.country);
    }

    private static Locale defaultLocale = new Locale(System.getProperty("microedition.locale"));

    public static Locale getDefaultLocale() {
      return defaultLocale;
    }

    public static void setDefaultLocale(Locale locale) {
      defaultLocale = locale;
    }

  }

}
