/*******************************************************************************
SiJaPP - Simple Java PreProcessor
Copyright (C) 2003  Manuel Linsmayer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*******************************************************************************/


package sijapp;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;


public class Preprocessor {


  // Target
  private String target;


  // Scanner
  private Scanner scanner = new Scanner();


  // Constructor
  public Preprocessor(String target) {
    this.target = new String(target);
  }


  // Returns the target
  public String getTarget() {
    return (new String(this.target));
  }


  // Preprocess
  public void run(BufferedReader reader, BufferedWriter writer) throws SijappException {

    // Reset line counter
    int lineNum = 1;

    boolean skip = false;
    boolean done = false;
    boolean ifAllowed = true;
    boolean elseifAllowed = false;
    boolean unlessAllowed = true;
    boolean elseAllowed = false;
    boolean endAllowed = false;

    // Read until EOF
    try {
      String line;
      while ((line = reader.readLine()) != null) {

        // Scan read line for a SiJaPP statement
        Scanner.Token[] tokens = this.scanner.scan(line);

        // No statement has been found
        if (tokens.length == 0) {
          if (!skip) {
            writer.write(line, 0, line.length());
            writer.newLine();
          }
        }
        // A statement has been found
        else {
          if (tokens.length == 3) {
            if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_EXIT) &&
                (tokens[2].getType() == Scanner.Token.T_MAGIC_END)) {
              if (!skip) {
                return;
              }
            }
            else {
              throw (new SijappException("Syntax error1"));
            }
          }
          else if (tokens.length == 4) {
            if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_ECHO) &&
                (tokens[2].getType() == Scanner.Token.T_STRING) &&
                (tokens[3].getType() == Scanner.Token.T_MAGIC_END)) {
              if (!skip) {
                String s = (String) tokens[2].getValue();
                writer.write(s, 0, s.length());
                writer.newLine();
              }
            }
            else {
              throw (new SijappException("Syntax error2"));
            }
          }
          else if (tokens.length == 5) {
            if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_TARGET) &&
                (tokens[2].getType() == Scanner.Token.T_CMDSEP) &&
                (tokens[3].getType() == Scanner.Token.T_CMD_TARGET_ELSE) &&
                (tokens[4].getType() == Scanner.Token.T_MAGIC_END) &&
                elseAllowed) {
              skip = done;
              ifAllowed = false;
              elseifAllowed = false;
              unlessAllowed = false;
              elseAllowed = false;
              endAllowed = true;
            }
            else if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                     (tokens[1].getType() == Scanner.Token.T_CMD_TARGET) &&
                     (tokens[2].getType() == Scanner.Token.T_CMDSEP) &&
                     (tokens[3].getType() == Scanner.Token.T_CMD_TARGET_END) &&
                     (tokens[4].getType() == Scanner.Token.T_MAGIC_END) &&
                     endAllowed) {
              skip = false;
              ifAllowed = true;
              elseifAllowed = false;
              unlessAllowed = true;
              elseAllowed = false;
              endAllowed = false;
            }
            else {
              throw (new SijappException("Syntax error3"));
            }
          }
          else if (tokens.length == 6) {
            if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_TARGET) &&
                (tokens[2].getType() == Scanner.Token.T_CMDSEP) &&
                (tokens[3].getType() == Scanner.Token.T_CMD_TARGET_IF) &&
                (tokens[4].getType() == Scanner.Token.T_IDENT) &&
                (tokens[5].getType() == Scanner.Token.T_MAGIC_END) &&
                ifAllowed) {
              if (this.target.equals((String) tokens[4].getValue())) {
                skip = false;
                done = true;
              }
              else {
                skip = true;
                done = false;
              }
              ifAllowed = false;
              elseifAllowed = true;
              unlessAllowed = false;
              elseAllowed = true;
              endAllowed = true;
            }
            else if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_TARGET) &&
                (tokens[2].getType() == Scanner.Token.T_CMDSEP) &&
                (tokens[3].getType() == Scanner.Token.T_CMD_TARGET_ELSEIF) &&
                (tokens[4].getType() == Scanner.Token.T_IDENT) &&
                (tokens[5].getType() == Scanner.Token.T_MAGIC_END) &&
                elseifAllowed) {
              if (!done && this.target.equals((String) tokens[4].getValue())) {
                skip = false;
                done = true;
              }
              else {
                skip = true;
                done = done;
              }
              ifAllowed = false;
              elseifAllowed = true;
              unlessAllowed = false;
              elseAllowed = true;
              endAllowed = true;
            }
            else if ((tokens[0].getType() == Scanner.Token.T_MAGIC_BEGIN) &&
                (tokens[1].getType() == Scanner.Token.T_CMD_TARGET) &&
                (tokens[2].getType() == Scanner.Token.T_CMDSEP) &&
                (tokens[3].getType() == Scanner.Token.T_CMD_TARGET_UNLESS) &&
                (tokens[4].getType() == Scanner.Token.T_IDENT) &&
                (tokens[5].getType() == Scanner.Token.T_MAGIC_END) &&
                unlessAllowed) {
              if (!this.target.equals((String) tokens[4].getValue())) {
                skip = false;
                done = true;
              }
              else {
                skip = true;
                done = false;
              }
              ifAllowed = false;
              elseifAllowed = false;
              unlessAllowed = false;
              elseAllowed = true;
              endAllowed = true;
            }
            else {
              throw (new SijappException("Syntax error4"));
            }

          }
          else {
            throw (new SijappException("Syntax error5"));
          }

        }

        // Advance line counter
        lineNum++;

      }
    }
    catch (SijappException e) {
      throw (new SijappException(lineNum + ": " + e.getMessage()));
    }
    catch (IOException e) {
      throw (new SijappException("An I/O error occured"));
    }

  }


}
